@@ -6,6 +6,9 @@ set -o errexit  # Exit the script with error if any of the commands fail
 #       AUTH                    Set to enable authentication. Defaults to "noauth"
 #       SSL                     Set to enable SSL. Defaults to "nossl"
 #       TOX_ENV                 Tox environment name, e.g. "tornado4-py36"
+#       TOX_BINARY              Path to tox executable
+#       INSTALL_TOX             Whether to install tox in a virtualenv
+#       PYTHON_BINARY           Path to python
 
 AUTH=${AUTH:-noauth}
 SSL=${SSL:-nossl}
@@ -24,5 +27,14 @@ if [ "$TOX_ENV" = "synchro" ]; then
     SETUP_ARGS="-- --check-exclude-patterns"
 fi
 
+if [ "${INSTALL_TOX}" = "true" ]; then
+    /opt/mongodbtoolchain/v2/bin/virtualenv motorenv
+    set +o xtrace
+    source motorenv/bin/activate
+    set -o xtrace
+    pip install tox
+    TOX_BINARY=tox
+fi
+
 # Run the tests, and store the results in Evergreen compatible XUnit XML
-${TOX_BINARY} -e ${TOX_ENV} ${SETUP_ARGS}
+${TOX_BINARY} -e ${TOX_ENV} ${SETUP_ARGS} "$@"
