@@ -343,10 +343,10 @@ If we have a collection with 2 documents like ``{'x': 1}``, then this code
 retrieves and deletes one of them::
 
   async def delete_one_document():
-      print(await db.test.count({'x': 1}))
+      print(await db.test.count_documents({'x': 1}))
       doc = await db.test.find_one_and_delete({'x': 1})
       print(doc)
-      print(await db.test.count({'x': 1}))
+      print(await db.test.count_documents({'x': 1}))
 
 This outputs something like::
 
@@ -596,7 +596,7 @@ metadata about the indexes, except for the ``"ns"`` and
 ``"name"`` keys, which are cleaned. For example::
 
   async def create_x_index():
-      print(await db.test.ensure_index("x", unique=True))
+      print(await db.test.create_index("x", unique=True))
       print(await db.test.index_information())
 
 This prints::
@@ -700,15 +700,13 @@ response from the server to the `map reduce command`_.
   - `session` (optional): a
     :class:`~pymongo.client_session.ClientSession`, created with
     :meth:`~MotorClient.start_session`.
-  - `callback` (optional): function taking (result, error), executed when
-    operation completes.
   - `**kwargs` (optional): additional arguments to the
     `map reduce command`_ may be passed as keyword arguments to this
     helper method, e.g.::
 
        result = yield db.test.map_reduce(map, reduce, "myresults", limit=2)
 
-If a callback is passed, returns None, else returns a Future.
+Returns a Future.
 
 .. note:: The :meth:`map_reduce` method does **not** obey the
    :attr:`read_preference` of this :class:`MotorCollection`. To run
@@ -947,8 +945,7 @@ To sort by multiple fields, pass a list of (key, direction) pairs:
   >>> IOLoop.current().run_sync(f)
   [(0, 4), (0, 2), (0, 0), (1, 3), (1, 1)]
 
-Beginning with MongoDB version 2.6, text search results can be
-sorted by relevance:
+Text search results can be sorted by relevance:
 
 .. testsetup:: sort_text
 
@@ -989,41 +986,71 @@ cursor has any effect.
     key, if not given :data:`~pymongo.ASCENDING` is assumed
 """
 
-start_session_doc = """
-Start a logical session.
+
+start_session_doc = """Start a logical session.
 
 This method takes the same parameters as PyMongo's
 :class:`~pymongo.client_session.SessionOptions`. See the
 :mod:`~pymongo.client_session` module for details.
 
+This session is created uninitialized, use it in an ``await`` expression
+to initialize it, or an ``async with`` statement.
+
 .. code-block:: python3
 
   async def coro():
       collection = client.db.collection
-  
-      with (await client.start_session()) as s:
+
+      # End the session after using it.
+      s = await client.start_session()
+      await s.end_session()
+
+      # Or, use an "async with" statement to end the session
+      # automatically.
+      async with await client.start_session() as s:
           doc = {'_id': ObjectId(), 'x': 1}
           await collection.insert_one(doc, session=s)
 
           secondary = collection.with_options(
               read_preference=ReadPreference.SECONDARY)
-  
-          # Sessions are causally consistent by default, we can read the doc
-          # we just inserted, even reading from a secondary. 
+
+          # Sessions are causally consistent by default, so we can read
+          # the doc we just inserted, even reading from a secondary.
           async for doc in secondary.find(session=s):
               print(doc)
+              
+      # Run a multi-document transaction:
+      async with await client.start_session() as s:
+          # Note, start_transaction doesn't require "await".
+          async with s.start_transaction():
+              await collection.delete_one({'x': 1}, session=s)
+              await collection.insert_one({'x': 2}, session=s)
+          
+          # Exiting the "with s.start_transaction()" block while throwing an
+          # exception automatically aborts the transaction, exiting the block
+          # normally automatically commits it.
 
+          # You can run additional transactions in the same session, so long as 
+          # you run them one at a time.
+          async with s.start_transaction():
+              await collection.insert_one({'x': 3}, session=s)
+              await collection.insert_many({'x': {'$gte': 2}},
+                                           {'$inc': {'x': 1}}, 
+                                           session=s)
+
+
+Requires MongoDB 3.6.
 Do **not** use the same session for multiple operations concurrently.
+A :class:`~MotorClientSession` may only be used with the MotorClient that
+started it.
 
-Requires MongoDB 3.6. It is an error to call :meth:`start_session`
-if this client has been authenticated to multiple databases using the
-deprecated method :meth:`~pymongo.database.Database.authenticate`.
-
-A :class:`~pymongo.client_session.ClientSession` may only be used with
-the MongoClient that started it.
-  
 :Returns:
-  An instance of :class:`~pymongo.client_session.ClientSession`.
+  An instance of :class:`~MotorClientSession`.
+
+.. versionchanged:: 2.0
+  Returns a :class:`~MotorClientSession`. Before, this
+  method returned a PyMongo
+  :class:`~pymongo.client_session.ClientSession`.
 
 .. versionadded:: 1.2
 """
