@@ -20,17 +20,12 @@ See "Frameworks" in the Developer Guide.
 
 import asyncio
 import asyncio.tasks
-import os
-
 import functools
 import multiprocessing
+import os
+from asyncio import coroutine  # For framework interface.
 from concurrent.futures import ThreadPoolExecutor
 
-try:
-    from asyncio import ensure_future
-except ImportError:
-    from asyncio import async as ensure_future
-
 CLASS_PREFIX = 'AsyncIO'
 
 
@@ -99,36 +94,37 @@ def run_on_executor(loop, fn, *args, **kwargs):
     return dest
 
 
-_DEFAULT = object()
+# Adapted from tornado.gen.
+def chain_future(a, b):
+    def copy(future):
+        assert future is a
+        if b.done():
+            return
+        if a.exception() is not None:
+            b.set_exception(a.exception())
+        else:
+            b.set_result(a.result())
+
+    a.add_done_callback(copy)
 
 
-def future_or_callback(future, callback, loop, return_value=_DEFAULT):
+def chain_return_value(future, loop, return_value):
     """Compatible way to return a value in all Pythons.
 
     PEP 479, raise StopIteration(value) from a coroutine won't work forever,
     but "return value" doesn't work in Python 2. Instead, Motor methods that
-    return values either execute a callback with the value or resolve a Future
-    with it, and are implemented with callbacks rather than a coroutine
-    internally.
+    return values resolve a Future with it, and are implemented with callbacks
+    rather than a coroutine internally.
     """
-    if callback:
-        raise NotImplementedError("Motor with asyncio prohibits callbacks")
-
-    if return_value is _DEFAULT:
-        return future
-
     chained = asyncio.Future(loop=loop)
 
-    def done_callback(_future):
-        try:
-            result = _future.result()
-            chained.set_result(result if return_value is _DEFAULT
-                               else return_value)
-        except Exception as exc:
-            chained.set_exception(exc)
+    def copy(_future):
+        if _future.exception() is not None:
+            chained.set_exception(_future.exception())
+        else:
+            chained.set_result(return_value)
 
-    future.add_done_callback(functools.partial(loop.call_soon_threadsafe,
-                                               done_callback))
+    future._future.add_done_callback(functools.partial(loop.add_callback, copy))
     return chained
 
 
@@ -148,9 +144,6 @@ def add_future(loop, future, callback, *args):
         functools.partial(loop.call_soon_threadsafe, callback, *args))
 
 
-coroutine = asyncio.coroutine
-
-
 def pymongo_class_wrapper(f, pymongo_class):
     """Executes the coroutine f and wraps its result in a Motor class.
 
@@ -172,5 +165,8 @@ def pymongo_class_wrapper(f, pymongo_class):
 
 
 def yieldable(future):
-    # TODO: really explain.
     return next(iter(future))
+
+
+def platform_info():
+    return None
