@@ -28,6 +28,9 @@ def create_asyncio_class(cls):
 AsyncIOMotorClient = create_asyncio_class(core.AgnosticClient)
 
 
+AsyncIOMotorClientSession = create_asyncio_class(core.AgnosticClientSession)
+
+
 AsyncIOMotorDatabase = create_asyncio_class(
     core.AgnosticDatabase)
 
@@ -52,14 +55,6 @@ AsyncIOMotorChangeStream = create_asyncio_class(
     core.AgnosticChangeStream)
 
 
-AsyncIOMotorBulkOperationBuilder = create_asyncio_class(
-    core.AgnosticBulkOperationBuilder)
-
-
-AsyncIOMotorGridFS = create_asyncio_class(
-    motor_gridfs.AgnosticGridFS)
-
-
 AsyncIOMotorGridFSBucket = create_asyncio_class(
     motor_gridfs.AgnosticGridFSBucket)
 
