@@ -30,6 +30,9 @@ def create_motor_class(cls):
 MotorClient = create_motor_class(core.AgnosticClient)
 
 
+MotorClientSession = create_motor_class(core.AgnosticClientSession)
+
+
 MotorDatabase = create_motor_class(core.AgnosticDatabase)
 
 
@@ -48,12 +51,6 @@ MotorLatentCommandCursor = create_motor_class(core.AgnosticLatentCommandCursor)
 MotorChangeStream = create_motor_class(core.AgnosticChangeStream)
 
 
-MotorBulkOperationBuilder = create_motor_class(core.AgnosticBulkOperationBuilder)
-
-
-MotorGridFS = create_motor_class(motor_gridfs.AgnosticGridFS)
-
-
 MotorGridFSBucket = create_motor_class(motor_gridfs.AgnosticGridFSBucket)
 
 
