@@ -21,13 +21,13 @@ PyMongo.
 DO NOT USE THIS MODULE.
 """
 
-import functools
 import inspect
 from tornado.ioloop import IOLoop
 
 import motor
 import motor.frameworks.tornado
 import motor.motor_tornado
+from motor.core import _MotorTransactionContext
 from motor.metaprogramming import MotorAttributeFactory
 
 # Make e.g. "from pymongo.errors import AutoReconnect" work. Note that
@@ -35,6 +35,7 @@ from motor.metaprogramming import MotorAttributeFactory
 from gridfs.errors import *
 from pymongo import *
 from pymongo import (collation,
+                     compression_support,
                      change_stream,
                      errors,
                      monotonic,
@@ -42,14 +43,18 @@ from pymongo import (collation,
                      server_selectors,
                      server_type,
                      son_manipulator,
+                     saslprep,
                      ssl_match_hostname,
                      ssl_support,
                      write_concern)
 from pymongo.auth import _build_credentials_tuple
 from pymongo.helpers import _check_command_response
+from pymongo.change_stream import _NON_RESUMABLE_GETMORE_ERRORS
+from pymongo.client_session import TransactionOptions
 from pymongo.collation import *
 from pymongo.common import *
 from pymongo.common import _UUID_REPRESENTATIONS, _MAX_END_SESSIONS
+from pymongo.compression_support import _HAVE_SNAPPY, _HAVE_ZLIB
 from pymongo.cursor import *
 from pymongo.cursor import _QUERY_OPTIONS
 from pymongo.errors import *
@@ -74,6 +79,7 @@ from pymongo.results import _WriteResult
 from pymongo.server import *
 from pymongo.server_selectors import *
 from pymongo.settings import *
+from pymongo.saslprep import *
 from pymongo.ssl_support import *
 from pymongo.son_manipulator import *
 from pymongo.topology import *
@@ -90,23 +96,16 @@ from pymongo import GEOSPHERE, HASHED
 from pymongo.pool import SocketInfo, Pool
 
 
-def unwrap_synchro(fn):
-    """If first argument to decorated function is a Synchro object, pass the
-    wrapped Motor object into the function.
-    """
-    @functools.wraps(fn)
-    def _unwrap_synchro(*args, **kwargs):
-        def _unwrap_obj(obj):
-            if isinstance(obj, Synchro):
-                return obj.delegate
-            else:
-                return obj
+# Internal classes not declared in motor_tornado.py: retrieve from class cache.
+from motor.core import (
+    AgnosticRawBatchCursor as _AgnosticRawBatchCursor,
+    AgnosticRawBatchCommandCursor as _AgnosticRawBatchCommandCursor)
+from motor.motor_tornado import create_motor_class
 
-        args = [_unwrap_obj(arg) for arg in args]
-        kwargs = dict([
-            (key, _unwrap_obj(value)) for key, value in kwargs.items()])
-        return fn(*args, **kwargs)
-    return _unwrap_synchro
+_MotorRawBatchCursor = create_motor_class(
+    _AgnosticRawBatchCursor)
+_MotorRawBatchCommandCursor = create_motor_class(
+    _AgnosticRawBatchCommandCursor)
 
 
 def wrap_synchro(fn):
@@ -123,19 +122,25 @@ def wrap_synchro(fn):
             client = MongoClient(delegate=motor_obj.database.client)
             database = Database(client, motor_obj.database.name)
             return Collection(database, motor_obj.name, delegate=motor_obj)
+        if isinstance(motor_obj, motor.motor_tornado.MotorClientSession):
+            return ClientSession(delegate=motor_obj)
+        if isinstance(motor_obj, _MotorTransactionContext):
+            return _SynchroTransactionContext(motor_obj)
         if isinstance(motor_obj, motor.MotorDatabase):
             client = MongoClient(delegate=motor_obj.client)
             return Database(client, motor_obj.name, delegate=motor_obj)
+        if isinstance(motor_obj, motor.motor_tornado.MotorChangeStream):
+            return ChangeStream(motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorLatentCommandCursor):
             return CommandCursor(motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorCommandCursor):
             return CommandCursor(motor_obj)
+        if isinstance(motor_obj, _MotorRawBatchCommandCursor):
+            return CommandCursor(motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorCursor):
             return Cursor(motor_obj)
-        if isinstance(motor_obj, motor.MotorBulkOperationBuilder):
-            return BulkOperationBuilder(motor_obj)
-        if isinstance(motor_obj, motor.MotorGridFS):
-            return GridFS(motor_obj)
+        if isinstance(motor_obj, _MotorRawBatchCursor):
+            return Cursor(motor_obj)
         if isinstance(motor_obj, motor.MotorGridIn):
             return GridIn(None, delegate=motor_obj)
         if isinstance(motor_obj, motor.MotorGridOut):
@@ -148,35 +153,53 @@ def wrap_synchro(fn):
     return _wrap_synchro
 
 
-class Sync(object):
-    def __init__(self, name):
+def unwrap_synchro(fn):
+    """Unwrap Synchro objects passed to a method and pass Motor objects instead.
+    """
+    @functools.wraps(fn)
+    def _unwrap_synchro(*args, **kwargs):
+        def _unwrap_obj(obj):
+            if isinstance(obj, Synchro):
+                return obj.delegate
+            else:
+                return obj
+
+        args = [_unwrap_obj(arg) for arg in args]
+        kwargs = dict([
+            (key, _unwrap_obj(value)) for key, value in kwargs.items()])
+        return fn(*args, **kwargs)
+    return _unwrap_synchro
+
+
+class SynchroAttr(object):
+    # Name can be set by SynchroMeta if Sync() is used directly in class defn.
+    def __init__(self, name=None):
         self.name = name
 
+
+class Sync(SynchroAttr):
     def __get__(self, obj, objtype):
         async_method = getattr(obj.delegate, self.name)
         return wrap_synchro(unwrap_synchro(obj.synchronize(async_method)))
 
 
-class WrapOutgoing(object):
+class WrapOutgoing(SynchroAttr):
     def __get__(self, obj, objtype):
         # self.name is set by SynchroMeta.
         name = self.name
 
         def synchro_method(*args, **kwargs):
             motor_method = getattr(obj.delegate, name)
-            return wrap_synchro(motor_method)(*args, **kwargs)
+            return wrap_synchro(unwrap_synchro(motor_method))(*args, **kwargs)
 
         return synchro_method
 
 
-class SynchroProperty(object):
+class SynchroProperty(SynchroAttr):
     """Used to fake private properties like MongoClient.__member - don't use
     for real properties like write_concern or you'll mask missing features in
     Motor!
     """
-    def __init__(self):
-        self.name = None
-
     def __get__(self, obj, objtype):
         # self.name is set by SynchroMeta.
         return getattr(obj.delegate.delegate, self.name)
@@ -227,7 +250,7 @@ class SynchroMeta(type):
 
             for attrname, delegate_attr in delegated_attrs.items():
                 # If attrname is in attrs, it means Synchro has overridden
-                # this attribute, e.g. Database.add_son_manipulator which is
+                # this attribute, e.g. Collection.aggregate which is
                 # special-cased. Ignore such attrs.
                 if attrname in attrs:
                     continue
@@ -236,7 +259,7 @@ class SynchroMeta(type):
                     # Re-synchronize the method.
                     setattr(new_class, attrname, Sync(attrname))
                 elif wrap_outgoing(delegate_attr):
-                    # Wrap MotorCursors in Synchro Cursors.
+                    # Wrap Motor objects in Synchro objects.
                     wrapper = WrapOutgoing()
                     wrapper.name = attrname
                     setattr(new_class, attrname, wrapper)
@@ -246,10 +269,9 @@ class SynchroMeta(type):
 
         # Set DelegateProperties' and SynchroProperties' names.
         for name, attr in attrs.items():
-            if isinstance(attr, (MotorAttributeFactory,
-                                 SynchroProperty,
-                                 WrapOutgoing)):
-                attr.name = name
+            if isinstance(attr, (MotorAttributeFactory, SynchroAttr)):
+                if attr.name is None:
+                    attr.name = name
 
         return new_class
 
@@ -288,9 +310,9 @@ class MongoClient(Synchro):
 
     _cache_credentials = SynchroProperty()
     get_database = WrapOutgoing()
-    get_default_database = WrapOutgoing()
     max_pool_size = SynchroProperty()
     max_write_batch_size = SynchroProperty()
+    start_session = Sync()
 
     def __init__(self, host=None, port=None, *args, **kwargs):
         # So that TestClient.test_constants and test_types work.
@@ -323,9 +345,44 @@ class MongoClient(Synchro):
     def __getitem__(self, name):
         return Database(self, name, delegate=self.delegate[name])
 
-    _MongoClient__options  = SynchroProperty()
-    _get_topology          = SynchroProperty()
-    _kill_cursors_executor = SynchroProperty()
+    _MongoClient__all_credentials = SynchroProperty()
+    _MongoClient__options         = SynchroProperty()
+    _get_topology                 = SynchroProperty()
+    _topology                     = SynchroProperty()
+    _kill_cursors_executor        = SynchroProperty()
+
+
+class _SynchroTransactionContext(Synchro):
+    def __init__(self, delegate):
+        self.delegate = delegate
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.synchronize(self.delegate._session.end_session)()
+
+
+class ClientSession(Synchro):
+    __delegate_class__ = motor.motor_tornado.MotorClientSession
+
+    start_transaction = WrapOutgoing()
+    client            = SynchroProperty()
+
+    def __init__(self, delegate):
+        self.delegate = delegate
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.synchronize(self.delegate.end_session)
+
+    _client              = SynchroProperty()
+    _in_transaction      = SynchroProperty()
+    _server_session      = SynchroProperty()
+    _transaction_id      = SynchroProperty()
+    _txn_read_preference = SynchroProperty()
 
 
 class Database(Synchro):
@@ -345,14 +402,6 @@ class Database(Synchro):
             "synchro.Database delegate must be MotorDatabase, not "
             " %s" % repr(self.delegate))
 
-    def add_son_manipulator(self, manipulator):
-        if isinstance(manipulator, son_manipulator.AutoReference):
-            db = manipulator.database
-            if isinstance(db, Database):
-                manipulator.database = db.delegate.delegate
-
-        self.delegate.add_son_manipulator(manipulator)
-
     @property
     def client(self):
         return self._client
@@ -368,9 +417,9 @@ class Collection(Synchro):
     __delegate_class__ = motor.MotorCollection
 
     find                            = WrapOutgoing()
-    initialize_unordered_bulk_op    = WrapOutgoing()
-    initialize_ordered_bulk_op      = WrapOutgoing()
+    find_raw_batches                = WrapOutgoing()
     list_indexes                    = WrapOutgoing()
+    watch                           = WrapOutgoing()
 
     def __init__(self, database, name, **kwargs):
         if not isinstance(database, Database):
@@ -389,13 +438,17 @@ class Collection(Synchro):
 
     def aggregate(self, *args, **kwargs):
         # Motor does no I/O initially in aggregate() but PyMongo does.
-        cursor = wrap_synchro(self.delegate.aggregate)(*args, **kwargs)
+        func = wrap_synchro(unwrap_synchro(self.delegate.aggregate))
+        cursor = func(*args, **kwargs)
         self.synchronize(cursor.delegate._get_more)()
         return cursor
 
-    def parallel_scan(self, *args, **kwargs):
-        # Motor does no I/O initially in parallel_scan() but PyMongo does.
-        return self.synchronize(self.delegate.parallel_scan)(*args, **kwargs)
+    def aggregate_raw_batches(self, *args, **kwargs):
+        # Motor does no I/O initially in aggregate() but PyMongo does.
+        func = wrap_synchro(unwrap_synchro(self.delegate.aggregate_raw_batches))
+        cursor = func(*args, **kwargs)
+        self.synchronize(cursor.delegate._get_more)()
+        return cursor
 
     def __getattr__(self, name):
         # Access to collections with dotted names, like db.test.mike
@@ -410,12 +463,29 @@ class Collection(Synchro):
                           delegate=self.delegate[name])
 
 
+class ChangeStream(Synchro):
+    __delegate_class__ = motor.motor_tornado.MotorChangeStream
+
+    next = Sync('next')
+    close = Sync('close')
+
+    def __init__(self, motor_change_stream):
+        self.delegate = motor_change_stream
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
+
 class Cursor(Synchro):
     __delegate_class__ = motor.motor_tornado.MotorCursor
 
-    rewind = WrapOutgoing()
-    clone  = WrapOutgoing()
-    close  = Sync('close')
+    batch_size = WrapOutgoing()
+    rewind     = WrapOutgoing()
+    clone      = WrapOutgoing()
+    close      = Sync('close')
 
     def __init__(self, motor_cursor):
         self.delegate = motor_cursor
@@ -473,9 +543,9 @@ class Cursor(Synchro):
     _Cursor__max_time_ms       = SynchroProperty()
     _Cursor__query_flags       = SynchroProperty()
     _Cursor__query_spec        = SynchroProperty()
-    _Cursor__read_preference   = SynchroProperty()
     _Cursor__retrieved         = SynchroProperty()
     _Cursor__spec              = SynchroProperty()
+    _read_preference           = SynchroProperty()
 
 
 class CommandCursor(Cursor):
@@ -505,45 +575,20 @@ class CursorManager(object):
     pass
 
 
-class BulkOperationBuilder(Synchro):
-    __delegate_class__ = motor.MotorBulkOperationBuilder
-
-    def __init__(self, motor_bob):
-        if not isinstance(motor_bob, motor.MotorBulkOperationBuilder):
-            raise TypeError(
-                "Expected MotorBulkOperationBuilder, got %r" % motor_bob)
-
-        self.delegate = motor_bob
-
-
-class GridFS(Synchro):
-    __delegate_class__ = motor.MotorGridFS
-
-    def __init__(self, database, collection='fs'):
-        if not isinstance(database, Database):
-            raise TypeError(
-                "Expected Database, got %s" % repr(database))
-
-        self.delegate = motor.MotorGridFS(database.delegate, collection)
-
-    def put(self, *args, **kwargs):
-        return self.synchronize(self.delegate.put)(*args, **kwargs)
-
-    def find(self, *args, **kwargs):
-        motor_method = self.delegate.find
-        unwrapping_method = wrap_synchro(unwrap_synchro(motor_method))
-        return unwrapping_method(*args, **kwargs)
+class BulkOperationBuilder(object):
+    pass
 
 
 class GridFSBucket(Synchro):
     __delegate_class__ = motor.MotorGridFSBucket
 
-    def __init__(self, database, bucket_name='fs'):
+    def __init__(self, database, bucket_name='fs', disable_md5=False):
         if not isinstance(database, Database):
             raise TypeError(
                 "Expected Database, got %s" % repr(database))
 
-        self.delegate = motor.MotorGridFSBucket(database.delegate, bucket_name)
+        self.delegate = motor.MotorGridFSBucket(
+            database.delegate, bucket_name, disable_md5)
 
     def find(self, *args, **kwargs):
         motor_method = self.delegate.find
