@@ -16,8 +16,12 @@ from __future__ import unicode_literals, absolute_import
 
 import warnings
 
-from motor.motor_asyncio import AsyncIOMotorGridFS
+import bson
+
+from motor.motor_asyncio import (AsyncIOMotorClientSession,
+                                 AsyncIOMotorGridFSBucket)
 import test
+from test import env
 from test.asyncio_tests import asyncio_test, AsyncIOTestCase
 
 
@@ -56,6 +60,11 @@ class TestAsyncIOAwait(AsyncIOTestCase):
 
             self.assertEqual(j, n_docs)
 
+            j = 0
+            raw_cursor = collection.find_raw_batches().sort('_id').batch_size(3)
+            async for batch in raw_cursor:
+                j += len(bson.decode_all(batch))
+
             await collection.delete_many({})
 
     @asyncio_test
@@ -75,18 +84,24 @@ class TestAsyncIOAwait(AsyncIOTestCase):
 
             # Force extra batches to test iteration.
             j = 0
-            async for doc in collection.aggregate(pipeline,
-                                                  cursor={'batchSize': 3}):
+            cursor = collection.aggregate(pipeline).batch_size(3)
+            async for doc in cursor:
                 self.assertEqual(j, doc['_id'])
                 j += 1
 
             self.assertEqual(j, n_docs)
 
+            j = 0
+            raw = collection.aggregate_raw_batches(pipeline).batch_size(3)
+            async for batch in raw:
+                j += len(bson.decode_all(batch))
+
+            self.assertEqual(j, n_docs)
             await collection.delete_many({})
 
     @asyncio_test
     async def test_iter_gridfs(self):
-        gfs = AsyncIOMotorGridFS(self.db)
+        gfs = AsyncIOMotorGridFSBucket(self.db)
 
         async def cleanup():
             await self.db.fs.files.delete_many({})
@@ -102,7 +117,8 @@ class TestAsyncIOAwait(AsyncIOTestCase):
 
         for n_files in 1, 2, 10:
             for i in range(n_files):
-                await gfs.put(data, filename='filename')
+                async with gfs.open_upload_stream(filename='filename') as f:
+                    await f.write(data)
 
             # Force extra batches to test iteration.
             j = 0
@@ -112,10 +128,11 @@ class TestAsyncIOAwait(AsyncIOTestCase):
             self.assertEqual(j, n_files)
             await cleanup()
 
-        async with await gfs.new_file(_id=1, chunk_size=1) as f:
-            await f.write(data)
-
-        gout = await gfs.find_one({'_id': 1})
+        await gfs.upload_from_stream_with_id(
+            1, 'filename', source=data, chunk_size_bytes=1)
+        cursor = gfs.find({'_id': 1})
+        await cursor.fetch_next
+        gout = cursor.next_object()
         chunks = []
         async for chunk in gout:
             chunks.append(chunk)
@@ -126,11 +143,12 @@ class TestAsyncIOAwait(AsyncIOTestCase):
     @asyncio_test
     async def test_stream_to_handler(self):
         # Sort of Tornado-specific, but it does work with asyncio.
-        fs = AsyncIOMotorGridFS(self.db)
+        fs = AsyncIOMotorGridFSBucket(self.db)
         content_length = 1000
         await fs.delete(1)
-        self.assertEqual(1, await fs.put(b'a' * content_length, _id=1))
-        gridout = await fs.get(1)
+        await fs.upload_from_stream_with_id(
+            1, 'filename', source=b'a' * content_length)
+        gridout = await fs.open_download_stream(1)
         handler = test.MockRequestHandler()
         await gridout.stream_to_handler(handler)
         self.assertEqual(content_length, handler.n_written)
@@ -157,3 +175,57 @@ class TestAsyncIOAwait(AsyncIOTestCase):
             keys.add(info['name'])
 
         self.assertEqual(keys, {'_id_', 'x_1', 'y_-1'})
+
+    @env.require_version_min(3, 6)
+    @env.require_replica_set
+    @asyncio_test
+    async def test_session(self):
+        s = await self.cx.start_session()
+        self.assertIsInstance(s, AsyncIOMotorClientSession)
+        self.assertIs(s.client, self.cx)
+        self.assertFalse(s.has_ended)
+        await s.end_session()
+        self.assertTrue(s.has_ended)
+
+        # Raises a helpful error if used in a regular with-statement.
+        with self.assertRaises(AttributeError) as ctx:
+            with await self.cx.start_session():
+                pass
+
+        self.assertIn("async with await", str(ctx.exception))
+
+        async with await self.cx.start_session() as s:
+            self.assertIsInstance(s, AsyncIOMotorClientSession)
+            self.assertFalse(s.has_ended)
+            await s.end_session()
+            self.assertTrue(s.has_ended)
+
+        self.assertTrue(s.has_ended)
+
+    @env.require_version_min(3, 7)
+    @env.require_replica_set
+    @asyncio_test
+    async def test_transaction(self):
+        async with await self.cx.start_session() as s:
+            s.start_transaction()
+            self.assertTrue(s.delegate._in_transaction)
+            self.assertFalse(s.has_ended)
+            await s.end_session()
+            self.assertFalse(s.delegate._in_transaction)
+            self.assertTrue(s.has_ended)
+
+        async with await self.cx.start_session() as s:
+            # Use start_transaction in "async with", not "async with await".
+            with self.assertRaises(TypeError):
+                async with await s.start_transaction():
+                    pass
+
+            await s.abort_transaction()
+
+            async with s.start_transaction():
+                self.assertTrue(s.delegate._in_transaction)
+                self.assertFalse(s.has_ended)
+            self.assertFalse(s.delegate._in_transaction)
+            self.assertFalse(s.has_ended)
+
+        self.assertTrue(s.has_ended)
