@@ -24,6 +24,7 @@ from bson import CodecOptions
 from pymongo import ReadPreference, WriteConcern
 from pymongo.errors import ConnectionFailure, OperationFailure
 
+import motor
 from motor import motor_asyncio
 
 import test
@@ -46,7 +47,7 @@ class TestAsyncIOClient(AsyncIOTestCase):
         future0 = collection.insert_one({'foo': 'bar'})
         future1 = collection.insert_one({'foo': 'bar'})
         yield from asyncio.gather(future0, future1, loop=self.loop)
-        resp = yield from collection.find({'foo': 'bar'}).count()
+        resp = yield from collection.count_documents({'foo': 'bar'})
         self.assertEqual(2, resp)
         cx.close()
 
@@ -67,12 +68,13 @@ class TestAsyncIOClient(AsyncIOTestCase):
             raise SkipTest("Socket file is not accessible")
 
         encoded_socket = '%2Ftmp%2Fmongodb-' + str(env.port) + '.sock'
-        uri = 'mongodb://%s' % encoded_socket
+        if test.env.auth:
+            uri = 'mongodb://%s:%s@%s' % (db_user, db_password, encoded_socket)
+        else:
+            uri = 'mongodb://%s' % (encoded_socket,)
+
         client = self.asyncio_client(uri)
         collection = client.motor_test.test
-
-        if test.env.auth:
-            yield from client.admin.authenticate(db_user, db_password)
         yield from collection.insert_one({"dummy": "object"})
 
         # Confirm it fails with a missing socket.
@@ -142,58 +144,16 @@ class TestAsyncIOClient(AsyncIOTestCase):
         self.assertEqual(cx.max_pool_size, 100)
         cx.close()
 
-    @asyncio_test(timeout=60)
-    def test_high_concurrency(self):
-        yield from self.make_test_data()
-
-        concurrency = 25
-        cx = self.asyncio_client(maxPoolSize=concurrency)
-        expected_finds = 200 * concurrency
-        n_inserts = 25
-
-        collection = cx.motor_test.test_collection
-        insert_collection = cx.motor_test.insert_collection
-        yield from insert_collection.delete_many({})
-
-        ndocs = 0
-        insert_future = asyncio.Future(loop=self.loop)
-
-        @asyncio.coroutine
-        def find():
-            nonlocal ndocs
-            cursor = collection.find()
-            while (yield from cursor.fetch_next):
-                cursor.next_object()
-                ndocs += 1
-
-                # Half-way through, start an insert loop
-                if ndocs == expected_finds / 2:
-                    asyncio.Task(insert(), loop=self.loop)
-
-        @asyncio.coroutine
-        def insert():
-            for i in range(n_inserts):
-                yield from insert_collection.insert_one({'s': hex(i)})
-
-            insert_future.set_result(None)  # Finished
-
-        yield from asyncio.gather(*[find() for _ in range(concurrency)],
-                                  loop=self.loop)
-        yield from insert_future
-        self.assertEqual(expected_finds, ndocs)
-        self.assertEqual(n_inserts, (yield from insert_collection.count()))
-        yield from collection.delete_many({})
-
     @asyncio_test(timeout=30)
     def test_drop_database(self):
         # Make sure we can pass an AsyncIOMotorDatabase instance
         # to drop_database
         db = self.cx.test_drop_database
         yield from db.test_collection.insert_one({})
-        names = yield from self.cx.database_names()
+        names = yield from self.cx.list_database_names()
         self.assertTrue('test_drop_database' in names)
         yield from self.cx.drop_database(db)
-        names = yield from self.cx.database_names()
+        names = yield from self.cx.list_database_names()
         self.assertFalse('test_drop_database' in names)
 
     @asyncio_test
@@ -205,9 +165,8 @@ class TestAsyncIOClient(AsyncIOTestCase):
         yield from remove_all_users(self.db)
         db = self.db
         try:
-            yield from db.add_user(
-                'mike', 'password',
-                roles=['userAdmin', 'readWrite'])
+            test.env.create_user(db.name, 'mike', 'password',
+                                 roles=['userAdmin', 'readWrite'])
 
             client = self.asyncio_client(
                 'mongodb://u:pass@%s:%d' % (env.host, env.port))
@@ -221,19 +180,7 @@ class TestAsyncIOClient(AsyncIOTestCase):
 
             yield from client[db.name].collection.find_one()
         finally:
-            yield from db.remove_user('mike')
-
-    @asyncio_test
-    def test_socketKeepAlive(self):
-        # Connect.
-        yield from self.cx.server_info()
-        ka = get_primary_pool(self.cx).opts.socket_keepalive
-        self.assertTrue(ka)
-
-        client = self.asyncio_client(socketKeepAlive=False)
-        yield from client.server_info()
-        ka = get_primary_pool(client).opts.socket_keepalive
-        self.assertFalse(ka)
+            test.env.drop_user(db.name, 'mike')
 
     def test_get_database(self):
         codec_options = CodecOptions(tz_aware=True)
@@ -283,5 +230,34 @@ class TestAsyncIOClientTimeout(AsyncIOMockServerTestCase):
         client.close()
 
 
+class TestAsyncIOClientHandshake(AsyncIOMockServerTestCase):
+    @asyncio_test
+    def test_handshake(self):
+        server = self.server()
+        client = motor_asyncio.AsyncIOMotorClient(server.uri,
+                                                  connectTimeoutMS=100,
+                                                  serverSelectionTimeoutMS=100)
+
+        # Trigger connection.
+        future = client.db.command('ping')
+        ismaster = yield from self.run_thread(server.receives, "ismaster")
+        meta = ismaster.doc['client']
+        self.assertEqual('PyMongo|Motor', meta['driver']['name'])
+        # AsyncIOMotorClient adds nothing to platform.
+        self.assertNotIn('Tornado', meta['platform'])
+        self.assertTrue(
+            meta['driver']['version'].endswith(motor.version),
+            "Version in handshake [%s] doesn't end with Motor version [%s]" % (
+                meta['driver']['version'], motor.version))
+
+        ismaster.hangs_up()
+        server.stop()
+        client.close()
+        try:
+            yield from future
+        except Exception:
+            pass
+
+
 if __name__ == '__main__':
     unittest.main()
