@@ -19,14 +19,16 @@ import sys
 import traceback
 import unittest
 import warnings
+from functools import partial
 from unittest import SkipTest
 
+import bson
 from pymongo import CursorType
 from pymongo.errors import InvalidOperation, ExecutionTimeout
 from pymongo.errors import OperationFailure
 
 from motor import motor_asyncio
-from test.utils import one, safe_get, get_primary_pool
+from test.utils import one, safe_get, get_primary_pool, TestListener
 from test.asyncio_tests import (asyncio_test,
                                 AsyncIOTestCase,
                                 AsyncIOMockServerTestCase,
@@ -44,13 +46,9 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
     def test_count(self):
         yield from self.make_test_data()
         coll = self.collection
-        self.assertEqual(200, (yield from coll.find().count()))
         self.assertEqual(
             100,
-            (yield from coll.find({'_id': {'$gt': 99}}).count()))
-
-        where = 'this._id % 2 == 0 && this._id >= 50'
-        self.assertEqual(75, (yield from coll.find().where(where).count()))
+            (yield from coll.count_documents({'_id': {'$gt': 99}})))
 
     @asyncio_test
     def test_fetch_next(self):
@@ -222,7 +220,7 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         collection = self.collection
         cursor = collection.find()
         docs = yield from cursor.to_list(None)  # Unlimited.
-        count = yield from collection.count()
+        count = yield from collection.count_documents({})
         self.assertEqual(count, len(docs))
 
     @asyncio_test
@@ -307,7 +305,8 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         cursor = collection.find()
         cursor.each(cancel)
         yield from future
-        self.assertEqual((yield from collection.count()), len(results))
+        self.assertEqual((yield from collection.count_documents({})),
+                         len(results))
 
     @asyncio_test
     def test_rewind(self):
@@ -413,7 +412,7 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
             None)
         self.assertEqual(1, len(socks))
         self.assertEqual(
-            (yield from self.db.test.count()),
+            (yield from self.db.test.count_documents({})),
             len(docs))
 
         # If the Cursor instance is discarded before being
@@ -452,6 +451,46 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
 
         self.assertEqual(0, len(w))
 
+    @asyncio_test
+    def test_aggregate_batch_size(self):
+        listener = TestListener()
+        cx = self.asyncio_client(event_listeners=[listener])
+        c = cx.motor_test.collection
+        yield from c.delete_many({})
+        yield from c.insert_many({'_id': i} for i in range(3))
+
+        # Two ways of setting batchSize.
+        cursor0 = c.aggregate([{'$sort': {'_id': 1}}]).batch_size(2)
+        cursor1 = c.aggregate([{'$sort': {'_id': 1}}], batchSize=2)
+        for cursor in cursor0, cursor1:
+            lst = []
+            while (yield from cursor.fetch_next):
+                lst.append(cursor.next_object())
+
+            self.assertEqual(lst, [{'_id': 0}, {'_id': 1}, {'_id': 2}])
+            aggregate = listener.first_command_started('aggregate')
+            self.assertEqual(aggregate.command['cursor']['batchSize'], 2)
+            getMore = listener.first_command_started('getMore')
+            self.assertEqual(getMore.command['batchSize'], 2)
+
+    @asyncio_test
+    def test_raw_batches(self):
+        c = self.collection
+        yield from c.delete_many({})
+        yield from c.insert_many({'_id': i} for i in range(4))
+
+        find = partial(c.find_raw_batches, {})
+        agg = partial(c.aggregate_raw_batches, [{'$sort': {'_id': 1}}])
+
+        for method in find, agg:
+            cursor = method().batch_size(2)
+            yield from cursor.fetch_next
+            batch = cursor.next_object()
+            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(batch))
+
+            lst = yield from method().batch_size(2).to_list(length=1)
+            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(lst[0]))
+
 
 class TestAsyncIOCursorMaxTimeMS(AsyncIOTestCase):
     def setUp(self):
