@@ -14,9 +14,7 @@
 
 """Test AsyncIOMotorDatabase."""
 
-import asyncio
 import unittest
-from unittest import SkipTest
 
 import pymongo.database
 from bson import CodecOptions
@@ -24,17 +22,9 @@ from bson.binary import JAVA_LEGACY
 from pymongo import ReadPreference, WriteConcern
 from pymongo.errors import CollectionInvalid, OperationFailure
 from pymongo.read_preferences import Secondary
-from pymongo.son_manipulator import NamespaceInjector, AutoReference
 
-from motor.motor_asyncio import (AsyncIOMotorDatabase,
-                                 AsyncIOMotorClient,
-                                 AsyncIOMotorCollection)
-import test
-from test import env
-from test.asyncio_tests import (asyncio_test,
-                                AsyncIOTestCase,
-                                remove_all_users)
-from test.utils import ignore_deprecations
+from motor.motor_asyncio import (AsyncIOMotorCollection, AsyncIOMotorDatabase)
+from test.asyncio_tests import (AsyncIOTestCase, asyncio_test)
 
 
 class TestAsyncIODatabase(AsyncIOTestCase):
@@ -81,7 +71,7 @@ class TestAsyncIODatabase(AsyncIOTestCase):
         collection = yield from db.create_collection('test_collection2')
         self.assertTrue(isinstance(collection, AsyncIOMotorCollection))
         self.assertTrue(
-            'test_collection2' in (yield from db.collection_names()))
+            'test_collection2' in (yield from db.list_collection_names()))
 
         with self.assertRaises(CollectionInvalid):
             yield from db.create_collection('test_collection2')
@@ -93,76 +83,12 @@ class TestAsyncIODatabase(AsyncIOTestCase):
         db = self.db
         collection = db.test_drop_collection
         yield from collection.insert_one({})
-        names = yield from db.collection_names()
+        names = yield from db.list_collection_names()
         self.assertTrue('test_drop_collection' in names)
         yield from db.drop_collection(collection)
-        names = yield from db.collection_names()
+        names = yield from db.list_collection_names()
         self.assertFalse('test_drop_collection' in names)
 
-    @ignore_deprecations
-    @asyncio_test
-    def test_auto_ref_and_deref(self):
-        # Test same functionality as in PyMongo's test_database.py; the
-        # implementation for Motor for async is a little complex so we test
-        # that it works here, and we don't just rely on synchrotest
-        # to cover it.
-        db = self.db
-
-        # We test a special hack where add_son_manipulator corrects our mistake
-        # if we pass an AsyncIOMotorDatabase, instead of Database, to
-        # AutoReference.
-        db.add_son_manipulator(AutoReference(db))
-        db.add_son_manipulator(NamespaceInjector())
-
-        a = {"hello": "world"}
-        b = {"test": a}
-        c = {"another test": b}
-
-        yield from db.a.delete_many({})
-        yield from db.b.delete_many({})
-        yield from db.c.delete_many({})
-        yield from db.a.save(a)
-        yield from db.b.save(b)
-        yield from db.c.save(c)
-        a["hello"] = "jesse"
-        yield from db.a.save(a)
-        result_a = yield from db.a.find_one()
-        result_b = yield from db.b.find_one()
-        result_c = yield from db.c.find_one()
-
-        self.assertEqual(a, result_a)
-        self.assertEqual(a, result_b["test"])
-        self.assertEqual(a, result_c["another test"]["test"])
-        self.assertEqual(b, result_b)
-        self.assertEqual(b, result_c["another test"])
-        self.assertEqual(c, result_c)
-
-    @asyncio_test
-    def test_authenticate(self):
-        if not test.env.auth:
-            raise SkipTest('Authentication is not enabled on server')
-
-        # self.db is logged in as root.
-        yield from self.db.add_user("jesse", "password")
-        db = AsyncIOMotorClient(env.host, env.port,
-                                **self.get_client_kwargs()).motor_test
-        try:
-            # Authenticate many times at once to test concurrency.
-            yield from asyncio.wait(
-                [db.authenticate("jesse", "password") for _ in range(10)],
-                loop=self.loop)
-
-            # Just make sure there are no exceptions here.
-            yield from db.remove_user("jesse")
-            yield from db.logout()
-            info = yield from self.db.command("usersInfo", "jesse")
-            users = info.get('users', [])
-            self.assertFalse("jesse" in [u['user'] for u in users])
-
-        finally:
-            yield from remove_all_users(self.db)
-            test.env.sync_cx.close()
-
     @asyncio_test
     def test_validate_collection(self):
         db = self.db
