@@ -23,6 +23,7 @@ from functools import wraps
 import pymongo.errors
 
 from test import SkipTest
+from test.utils import create_user
 from test.version import Version
 
 HAVE_SSL = True
@@ -220,7 +221,7 @@ class TestEnvironment(object):
         if db_user or db_password:
             if not (db_user and db_password):
                 sys.stderr.write(
-                    "You msut set both DB_USER and DB_PASSWORD, or neither\n")
+                    "You must set both DB_USER and DB_PASSWORD, or neither\n")
                 sys.exit(1)
 
             self.auth = True
@@ -252,40 +253,55 @@ class TestEnvironment(object):
         if self.sync_cx.server_info().get('javascriptEngine') == 'V8':
             self.v8 = True
 
-    def _require(self, condition, msg, func=None):
+    def require(self, condition, msg, func=None):
         def make_wrapper(f):
             @wraps(f)
             def wrap(*args, **kwargs):
                 if condition():
                     return f(*args, **kwargs)
                 raise SkipTest(msg)
+
             return wrap
 
         if func is None:
             def decorate(f):
                 return make_wrapper(f)
+
             return decorate
         return make_wrapper(func)
 
+    def require_auth(self, func):
+        """Run a test only if the server is started with auth."""
+        return self.require(
+            lambda: self.auth, "Server must be start with auth", func=func)
+
     def require_version_min(self, *ver):
         """Run a test only if the server version is at least ``version``."""
         other_version = Version(*ver)
-        return self._require(lambda: self.version >= other_version,
-                             "Server version must be at least %s"
-                             % str(other_version))
+        return self.require(lambda: self.version >= other_version,
+                            "Server version must be at least %s"
+                            % str(other_version))
 
     def require_version_max(self, *ver):
         """Run a test only if the server version is at most ``version``."""
         other_version = Version(*ver)
-        return self._require(lambda: self.version <= other_version,
-                             "Server version must be at most %s"
-                             % str(other_version))
+        return self.require(lambda: self.version <= other_version,
+                            "Server version must be at most %s"
+                            % str(other_version))
 
     def require_replica_set(self, func):
         """Run a test only if the client is connected to a replica set."""
-        return self._require(lambda: self.is_replica_set,
-                             "Not connected to a replica set",
-                             func=func)
+        return self.require(lambda: self.is_replica_set,
+                            "Not connected to a replica set",
+                            func=func)
+
+    def create_user(self, dbname, user, pwd=None, roles=None, **kwargs):
+        kwargs['writeConcern'] = {'w': self.w}
+        return create_user(self.sync_cx[dbname], user, pwd, roles, **kwargs)
+
+    def drop_user(self, dbname, user):
+        self.sync_cx[dbname].command(
+            'dropUser', user, writeConcern={'w': self.w})
 
 
 env = TestEnvironment()
