@@ -16,13 +16,17 @@ from __future__ import unicode_literals, absolute_import
 
 import warnings
 
+import bson
+
+from test import env
+
 """Test Motor, an asynchronous driver for MongoDB and Tornado."""
 
 from tornado.testing import gen_test
 
-from motor import MotorGridFS
+from motor.motor_tornado import MotorClientSession, MotorGridFSBucket
+
 import test
-from test import SkipTest
 from test.tornado_tests import MotorTest
 
 
@@ -61,6 +65,12 @@ class MotorTestAwait(MotorTest):
 
             self.assertEqual(j, n_docs)
 
+            j = 0
+            raw_cursor = collection.find_raw_batches().sort('_id').batch_size(3)
+            async for batch in raw_cursor:
+                j += len(bson.decode_all(batch))
+
+            self.assertEqual(j, n_docs)
             await collection.delete_many({})
 
     @gen_test
@@ -80,18 +90,24 @@ class MotorTestAwait(MotorTest):
 
             # Force extra batches to test iteration.
             j = 0
-            async for doc in collection.aggregate(pipeline,
-                                                  cursor={'batchSize': 3}):
+            cursor = collection.aggregate(pipeline).batch_size(3)
+            async for doc in cursor:
                 self.assertEqual(j, doc['_id'])
                 j += 1
 
             self.assertEqual(j, n_docs)
 
+            j = 0
+            raw = collection.aggregate_raw_batches(pipeline).batch_size(3)
+            async for batch in raw:
+                j += len(bson.decode_all(batch))
+
+            self.assertEqual(j, n_docs)
             await collection.delete_many({})
 
     @gen_test
     async def test_iter_gridfs(self):
-        gfs = MotorGridFS(self.db)
+        gfs = MotorGridFSBucket(self.db)
 
         async def cleanup():
             await self.db.fs.files.delete_many({})
@@ -107,7 +123,8 @@ class MotorTestAwait(MotorTest):
 
         for n_files in 1, 2, 10:
             for i in range(n_files):
-                await gfs.put(data, filename='filename')
+                async with gfs.open_upload_stream(filename='filename') as f:
+                    await f.write(data)
 
             # Force extra batches to test iteration.
             j = 0
@@ -117,10 +134,11 @@ class MotorTestAwait(MotorTest):
             self.assertEqual(j, n_files)
             await cleanup()
 
-        async with await gfs.new_file(_id=1, chunk_size=1) as f:
-            await f.write(data)
-
-        gout = await gfs.find_one({'_id': 1})
+        await gfs.upload_from_stream_with_id(
+            1, 'filename', source=data, chunk_size_bytes=1)
+        cursor = gfs.find({'_id': 1})
+        await cursor.fetch_next
+        gout = cursor.next_object()
         chunks = []
         async for chunk in gout:
             chunks.append(chunk)
@@ -130,11 +148,12 @@ class MotorTestAwait(MotorTest):
 
     @gen_test
     async def test_stream_to_handler(self):
-        fs = MotorGridFS(self.db)
+        fs = MotorGridFSBucket(self.db)
         content_length = 1000
         await fs.delete(1)
-        self.assertEqual(1, await fs.put(b'a' * content_length, _id=1))
-        gridout = await fs.get(1)
+        await fs.upload_from_stream_with_id(
+            1, 'filename', source=b'a' * content_length)
+        gridout = await fs.open_download_stream(1)
         handler = test.MockRequestHandler()
         await gridout.stream_to_handler(handler)
         self.assertEqual(content_length, handler.n_written)
@@ -150,3 +169,68 @@ class MotorTestAwait(MotorTest):
 
         if w:
             self.fail(w[0].message)
+
+    @gen_test
+    async def test_list_indexes(self):
+        await self.collection.drop()
+        await self.collection.create_index([('x', 1)])
+        await self.collection.create_index([('y', -1)])
+        keys = set()
+        async for info in self.collection.list_indexes():
+            keys.add(info['name'])
+
+        self.assertEqual(keys, {'_id_', 'x_1', 'y_-1'})
+
+    @env.require_version_min(3, 6)
+    @env.require_replica_set
+    @gen_test
+    async def test_session(self):
+        s = await self.cx.start_session()
+        self.assertIsInstance(s, MotorClientSession)
+        self.assertIs(s.client, self.cx)
+        self.assertFalse(s.has_ended)
+        await s.end_session()
+        self.assertTrue(s.has_ended)
+
+        # Raises a helpful error if used in a regular with-statement.
+        with self.assertRaises(AttributeError) as ctx:
+            with await self.cx.start_session():
+                pass
+
+        self.assertIn("async with await", str(ctx.exception))
+
+        async with await self.cx.start_session() as s:
+            self.assertIsInstance(s, MotorClientSession)
+            self.assertFalse(s.has_ended)
+            await s.end_session()
+            self.assertTrue(s.has_ended)
+
+        self.assertTrue(s.has_ended)
+
+    @env.require_version_min(3, 7)
+    @env.require_replica_set
+    @gen_test
+    async def test_transaction(self):
+        async with await self.cx.start_session() as s:
+            s.start_transaction()
+            self.assertTrue(s.delegate._in_transaction)
+            self.assertFalse(s.has_ended)
+            await s.end_session()
+            self.assertFalse(s.delegate._in_transaction)
+            self.assertTrue(s.has_ended)
+
+        async with await self.cx.start_session() as s:
+            # Use start_transaction in "async with", not "async with await".
+            with self.assertRaises(TypeError):
+                async with await s.start_transaction():
+                    pass
+
+            await s.abort_transaction()
+
+            async with s.start_transaction():
+                self.assertTrue(s.delegate._in_transaction)
+                self.assertFalse(s.has_ended)
+            self.assertFalse(s.delegate._in_transaction)
+            self.assertFalse(s.has_ended)
+
+        self.assertTrue(s.has_ended)
