@@ -53,7 +53,7 @@ class MotorClientTest(MotorTest):
         future1 = collection.insert_one({'foo': 'bar'})
         yield [future0, future1]
 
-        self.assertEqual(2, (yield collection.find({'foo': 'bar'}).count()))
+        self.assertEqual(2, (yield collection.count_documents({'foo': 'bar'})))
 
         cx.close()
 
@@ -67,12 +67,12 @@ class MotorClientTest(MotorTest):
             raise SkipTest("Socket file is not accessible")
 
         encoded_socket = '%2Ftmp%2Fmongodb-' + str(env.port) + '.sock'
-        uri = 'mongodb://%s' % encoded_socket
-        client = self.motor_client(uri)
-
         if test.env.auth:
-            yield client.admin.authenticate(db_user, db_password)
+            uri = 'mongodb://%s:%s@%s' % (db_user, db_password, encoded_socket)
+        else:
+            uri = 'mongodb://%s' % (encoded_socket,)
 
+        client = self.motor_client(uri)
         yield client.motor_test.test.insert_one({"dummy": "object"})
 
         # Confirm it fails with a missing socket.
@@ -99,15 +99,9 @@ class MotorClientTest(MotorTest):
         client = motor.MotorClient('localhost', 8765, io_loop=self.io_loop,
                                    serverSelectionTimeoutMS=10)
 
-        # Test the Future interface.
         with self.assertRaises(ConnectionFailure):
             yield client.admin.command('ismaster')
 
-        # Test with a callback.
-        (result, error), _ = yield gen.Task(client.admin.command, 'ismaster')
-        self.assertEqual(None, result)
-        self.assertTrue(isinstance(error, ConnectionFailure))
-
     @gen_test(timeout=30)
     def test_connection_timeout(self):
         # Motor merely tries to time out a connection attempt within the
@@ -132,55 +126,15 @@ class MotorClientTest(MotorTest):
         self.assertEqual(cx.max_pool_size, 100)
         cx.close()
 
-    @gen_test(timeout=60)
-    def test_high_concurrency(self):
-        yield self.make_test_data()
-
-        concurrency = 25
-        cx = self.motor_client(maxPoolSize=concurrency)
-        expected_finds = 200 * concurrency
-        n_inserts = 25
-
-        collection = cx.motor_test.test_collection
-        insert_collection = cx.motor_test.insert_collection
-        yield insert_collection.delete_many({})
-
-        ndocs = [0]
-        insert_future = Future()
-
-        @gen.coroutine
-        def find():
-            cursor = collection.find()
-            while (yield cursor.fetch_next):
-                cursor.next_object()
-                ndocs[0] += 1
-
-                # Half-way through, start an insert loop
-                if ndocs[0] == expected_finds / 2:
-                    insert()
-
-        @gen.coroutine
-        def insert():
-            for i in range(n_inserts):
-                yield insert_collection.insert_one({'s': hex(i)})
-
-            insert_future.set_result(None)  # Finished
-
-        yield [find() for _ in range(concurrency)]
-        yield insert_future
-        self.assertEqual(expected_finds, ndocs[0])
-        self.assertEqual(n_inserts, (yield insert_collection.count()))
-        yield collection.delete_many({})
-
     @gen_test(timeout=30)
     def test_drop_database(self):
         # Make sure we can pass a MotorDatabase instance to drop_database
         db = self.cx.test_drop_database
         yield db.test_collection.insert_one({})
-        names = yield self.cx.database_names()
+        names = yield self.cx.list_database_names()
         self.assertTrue('test_drop_database' in names)
         yield self.cx.drop_database(db)
-        names = yield self.cx.database_names()
+        names = yield self.cx.list_database_names()
         self.assertFalse('test_drop_database' in names)
 
     @gen_test
@@ -192,9 +146,8 @@ class MotorClientTest(MotorTest):
         yield remove_all_users(self.db)
         db = self.db
         try:
-            yield db.add_user(
-                'mike', 'password',
-                roles=['userAdmin', 'readWrite'])
+            test.env.create_user(db.name, 'mike', 'password',
+                                 roles=['userAdmin', 'readWrite'])
 
             client = self.motor_client(
                 'mongodb://u:pass@%s:%d' % (env.host, env.port))
@@ -208,7 +161,7 @@ class MotorClientTest(MotorTest):
 
             yield client[db.name].collection.find_one()
         finally:
-            yield db.remove_user('mike')
+            test.env.drop_user(db.name, 'mike')
 
     def test_get_database(self):
         codec_options = CodecOptions(tz_aware=True)
@@ -338,5 +291,33 @@ class MotorClientExhaustCursorTest(MotorMockServerTest):
         yield self._test_exhaust_query_network_error(rs=True)
 
 
+class MotorClientHandshakeTest(MotorMockServerTest):
+    @gen_test
+    def test_handshake(self):
+        server = self.server()
+        client = motor.MotorClient(server.uri,
+                                   connectTimeoutMS=100,
+                                   serverSelectionTimeoutMS=100)
+
+        # Trigger connection.
+        future = client.db.command('ping')
+        ismaster = yield self.run_thread(server.receives, "ismaster")
+        meta = ismaster.doc['client']
+        self.assertEqual('PyMongo|Motor', meta['driver']['name'])
+        self.assertIn('Tornado', meta['platform'])
+        self.assertTrue(
+            meta['driver']['version'].endswith(motor.version),
+            "Version in handshake [%s] doesn't end with Motor version [%s]" % (
+                meta['driver']['version'], motor.version))
+
+        ismaster.hangs_up()
+        server.stop()
+        client.close()
+        try:
+            yield future
+        except Exception:
+            pass
+
+
 if __name__ == '__main__':
     unittest.main()
