@@ -17,9 +17,9 @@ from __future__ import unicode_literals
 """Validate list of PyMongo attributes wrapped by Motor."""
 
 from tornado.testing import gen_test
-from gridfs import GridFS, GridIn
+from gridfs import GridFSBucket, GridIn
 
-from motor import MotorGridFS, MotorGridIn, MotorGridOut
+from motor import MotorGridFSBucket, MotorGridIn
 from test import env
 from test.tornado_tests import MotorTest
 
@@ -39,15 +39,39 @@ pymongo_only = set(['next'])
 motor_client_only = motor_only.union(['open'])
 
 pymongo_client_only = set([
+    'close_cursor',
+    'database_names',
     'is_locked',
-    'set_cursor_manager']).union(pymongo_only)
+    'set_cursor_manager',
+    'get_default_database',
+    'kill_cursors']).union(pymongo_only)
 
 pymongo_database_only = set([
-    'system_js']).union(pymongo_only)
+    'add_user',
+    'collection_names',
+    'remove_user',
+    'system_js',
+    'last_status',
+    'reset_error_history',
+    'eval',
+    'add_son_manipulator',
+    'logout',
+    'error',
+    'authenticate',
+    'previous_error']).union(pymongo_only)
 
 pymongo_collection_only = set([
-    'aggregate_raw_batches',
-    'find_raw_batches']).union(pymongo_only)
+    'count',
+    'ensure_index',
+    'group',
+    'initialize_ordered_bulk_op',
+    'initialize_unordered_bulk_op',
+    'save',
+    'remove',
+    'insert',
+    'update',
+    'find_and_modify',
+    'parallel_scan']).union(pymongo_only)
 
 motor_cursor_only = set([
     'fetch_next',
@@ -57,7 +81,9 @@ motor_cursor_only = set([
     'next_object',
     'closed']).union(motor_only)
 
-pymongo_cursor_only = set(['retrieved']).union(pymongo_only)
+pymongo_cursor_only = set([
+    'count',
+    'retrieved']).union(pymongo_only)
 
 
 class MotorCoreTest(MotorTest):
@@ -66,6 +92,14 @@ class MotorCoreTest(MotorTest):
             attrs(env.sync_cx) - pymongo_client_only,
             attrs(self.cx) - motor_client_only)
 
+    @env.require_version_min(3, 6)
+    @env.require_replica_set
+    @gen_test
+    def test_client_session_attrs(self):
+        self.assertEqual(
+            attrs(env.sync_cx.start_session()),
+            attrs((yield self.cx.start_session())) - motor_only)
+
     def test_database_attrs(self):
         self.assertEqual(
             attrs(env.sync_cx.test) - pymongo_database_only,
@@ -110,25 +144,19 @@ class MotorCoreTest(MotorTest):
 class MotorCoreTestGridFS(MotorTest):
     def setUp(self):
         super(MotorCoreTestGridFS, self).setUp()
-        self.sync_fs = GridFS(env.sync_cx.test)
-        self.sync_fs.delete(file_id=1)
-        self.sync_fs.put(b'', _id=1)
+        self.sync_fs = GridFSBucket(env.sync_cx.test)
+        self.sync_fs.upload_from_stream_with_id(1, 'filename', source=b'')
 
     def tearDown(self):
         self.sync_fs.delete(file_id=1)
         super(MotorCoreTestGridFS, self).tearDown()
 
     def test_gridfs_attrs(self):
-        pymongo_gridfs_only = set([
-            # Obsolete PyMongo methods.
-            'open',
-            'remove'])
-
         motor_gridfs_only = set(['collection']).union(motor_only)
 
         self.assertEqual(
-            attrs(GridFS(env.sync_cx.test)) - pymongo_gridfs_only,
-            attrs(MotorGridFS(self.cx.test)) - motor_gridfs_only)
+            attrs(GridFSBucket(env.sync_cx.test)),
+            attrs(MotorGridFSBucket(self.cx.test)) - motor_gridfs_only)
 
     def test_gridin_attrs(self):
         motor_gridin_only = set(['set']).union(motor_only)
@@ -144,12 +172,13 @@ class MotorCoreTestGridFS(MotorTest):
             'stream_to_handler'
         ]).union(motor_only)
 
-        motor_gridout = yield MotorGridOut(self.cx.test.fs, file_id=1).open()
+        fs = MotorGridFSBucket(self.cx.test)
+        motor_gridout = yield fs.open_download_stream(1)
         self.assertEqual(
-            attrs(self.sync_fs.get(1)),
+            attrs(self.sync_fs.open_download_stream(1)),
             attrs(motor_gridout) - motor_gridout_only)
 
     def test_gridout_cursor_attrs(self):
         self.assertEqual(
             attrs(self.sync_fs.find()) - pymongo_cursor_only,
-            attrs(MotorGridFS(self.cx.test).find()) - motor_cursor_only)
+            attrs(MotorGridFSBucket(self.cx.test).find()) - motor_cursor_only)
