@@ -20,7 +20,9 @@ import sys
 import traceback
 import unittest
 import warnings
+from functools import partial
 
+import bson
 import pymongo
 from tornado import gen
 from tornado.concurrent import Future
@@ -37,7 +39,7 @@ from test.tornado_tests import (get_command_line,
                                 MotorTest,
                                 MotorMockServerTest,
                                 server_is_mongos)
-from test.utils import one, safe_get, get_primary_pool
+from test.utils import one, safe_get, get_primary_pool, TestListener
 
 
 class MotorCursorTest(MotorMockServerTest):
@@ -50,18 +52,8 @@ class MotorCursorTest(MotorMockServerTest):
     def test_count(self):
         yield self.make_test_data()
         coll = self.collection
-        self.assertEqual(200, (yield coll.find().count()))
-        self.assertEqual(100, (yield coll.find({'_id': {'$gt': 99}}).count()))
-        where = 'this._id % 2 == 0 && this._id >= 50'
-        self.assertEqual(75, (yield coll.find({'$where': where}).count()))
-        self.assertEqual(75, (yield coll.find().where(where).count()))
-        self.assertEqual(
-            25,
-            (yield coll.find({'_id': {'$lt': 100}}).where(where).count()))
-
-        self.assertEqual(
-            25,
-            (yield coll.find({'_id': {'$lt': 100}, '$where': where}).count()))
+        self.assertEqual(100,
+                         (yield coll.count_documents({'_id': {'$gt': 99}})))
 
     @gen_test
     def test_fetch_next(self):
@@ -199,20 +191,6 @@ class MotorCursorTest(MotorMockServerTest):
         with self.assertRaises(TypeError):
             yield cursor.to_list('foo')
 
-    @gen_test
-    def test_to_list_callback(self):
-        yield self.make_test_data()
-        cursor = self.collection.find({}, {'_id': 1})
-        cursor.sort([('_id', pymongo.ASCENDING)])
-        expected = [{'_id': i} for i in range(200)]
-        (result, error), _ = yield gen.Task(cursor.to_list, length=1000)
-        self.assertEqual(expected, result)
-
-        cursor = self.collection.find().where('return foo')
-        (result, error), _ = yield gen.Task(cursor.to_list, length=1000)
-        self.assertEqual(None, result)
-        self.assertTrue(isinstance(error, OperationFailure))
-
     @gen_test
     def test_to_list_with_length(self):
         yield self.make_test_data()
@@ -267,7 +245,7 @@ class MotorCursorTest(MotorMockServerTest):
         collection = self.collection
         cursor = collection.find()
         docs = yield cursor.to_list(None)  # Unlimited.
-        count = yield collection.count()
+        count = yield collection.count_documents({})
         self.assertEqual(count, len(docs))
 
     @gen_test
@@ -358,7 +336,7 @@ class MotorCursorTest(MotorMockServerTest):
         cursor = collection.find()
         cursor.each(cancel)
         yield future
-        self.assertEqual((yield collection.count()), len(results))
+        self.assertEqual((yield collection.count_documents({})), len(results))
 
     @gen_test
     def test_rewind(self):
@@ -464,7 +442,7 @@ class MotorCursorTest(MotorMockServerTest):
             cursor_type=CursorType.EXHAUST).to_list(None)
         self.assertEqual(1, len(socks))
         self.assertEqual(
-            (yield self.db.test.count()),
+            (yield self.db.test.count_documents({})),
             len(docs))
 
         # If the Cursor instance is discarded before being
@@ -509,6 +487,46 @@ class MotorCursorTest(MotorMockServerTest):
 
         self.assertEqual(0, len(w))
 
+    @gen_test
+    def test_aggregate_batch_size(self):
+        listener = TestListener()
+        cx = self.motor_client(event_listeners=[listener])
+        c = cx.motor_test.collection
+        yield c.delete_many({})
+        yield c.insert_many({'_id': i} for i in range(3))
+
+        # Two ways of setting batchSize.
+        cursor0 = c.aggregate([{'$sort': {'_id': 1}}]).batch_size(2)
+        cursor1 = c.aggregate([{'$sort': {'_id': 1}}], batchSize=2)
+        for cursor in cursor0, cursor1:
+            lst = []
+            while (yield cursor.fetch_next):
+                lst.append(cursor.next_object())
+
+            self.assertEqual(lst, [{'_id': 0}, {'_id': 1}, {'_id': 2}])
+            aggregate = listener.first_command_started('aggregate')
+            self.assertEqual(aggregate.command['cursor']['batchSize'], 2)
+            getMore = listener.first_command_started('getMore')
+            self.assertEqual(getMore.command['batchSize'], 2)
+
+    @gen_test
+    def test_raw_batches(self):
+        c = self.collection
+        yield c.delete_many({})
+        yield c.insert_many({'_id': i} for i in range(4))
+
+        find = partial(c.find_raw_batches, {})
+        agg = partial(c.aggregate_raw_batches, [{'$sort': {'_id': 1}}])
+
+        for method in find, agg:
+            cursor = method().batch_size(2)
+            yield cursor.fetch_next
+            batch = cursor.next_object()
+            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(batch))
+
+            lst = yield method().batch_size(2).to_list(length=1)
+            self.assertEqual([{'_id': 0}, {'_id': 1}], bson.decode_all(lst[0]))
+
 
 class MotorCursorMaxTimeMSTest(MotorTest):
     def setUp(self):
