@@ -24,15 +24,11 @@ from bson.binary import JAVA_LEGACY
 from pymongo import ReadPreference, WriteConcern
 from pymongo.read_preferences import Secondary
 from pymongo.errors import OperationFailure, CollectionInvalid
-from pymongo.son_manipulator import AutoReference, NamespaceInjector
 from tornado import gen
 from tornado.testing import gen_test
 
 import motor
-import test
-from test.test_environment import env
-from test.tornado_tests import MotorTest, remove_all_users
-from test.utils import ignore_deprecations
+from test.tornado_tests import MotorTest
 
 
 class MotorDatabaseTest(MotorTest):
@@ -63,23 +59,6 @@ class MotorDatabaseTest(MotorTest):
         else:
             self.fail('Expected TypeError')
 
-    @gen_test
-    def test_database_callbacks(self):
-        db = self.db
-        yield db.drop_collection('c')
-
-        self.assertRaises(TypeError, db.create_collection, 'c', callback='foo')
-        self.assertRaises(TypeError, db.create_collection, 'c', callback=1)
-
-        # No error without callback
-        db.create_collection('c', callback=None)
-
-        # Wait for create_collection to complete
-        for _ in range(10):
-            yield gen.sleep(0.1)
-            if 'c' in (yield db.collection_names()):
-                break
-
     @gen_test
     def test_command(self):
         result = yield self.cx.admin.command("buildinfo")
@@ -95,7 +74,7 @@ class MotorDatabaseTest(MotorTest):
         collection = yield db.create_collection('test_collection2')
         self.assertTrue(isinstance(collection, motor.MotorCollection))
         self.assertTrue(
-            'test_collection2' in (yield db.collection_names()))
+            'test_collection2' in (yield db.list_collection_names()))
 
         with self.assertRaises(CollectionInvalid):
             yield db.create_collection('test_collection2')
@@ -118,74 +97,12 @@ class MotorDatabaseTest(MotorTest):
         db = self.db
         collection = db.test_drop_collection
         yield collection.insert_one({})
-        names = yield db.collection_names()
+        names = yield db.list_collection_names()
         self.assertTrue('test_drop_collection' in names)
         yield db.drop_collection(collection)
-        names = yield db.collection_names()
+        names = yield db.list_collection_names()
         self.assertFalse('test_drop_collection' in names)
 
-    @ignore_deprecations
-    @gen_test
-    def test_auto_ref_and_deref(self):
-        # Test same functionality as in PyMongo's test_database.py; the
-        # implementation for Motor for async is a little complex so we test
-        # that it works here, and we don't just rely on synchrotest
-        # to cover it.
-        db = self.db
-
-        # We test a special hack where add_son_manipulator corrects our mistake
-        # if we pass a MotorDatabase, instead of Database, to AutoReference.
-        db.add_son_manipulator(AutoReference(db))
-        db.add_son_manipulator(NamespaceInjector())
-
-        a = {"hello": "world"}
-        b = {"test": a}
-        c = {"another test": b}
-
-        yield db.a.delete_many({})
-        yield db.b.delete_many({})
-        yield db.c.delete_many({})
-        yield db.a.save(a)
-        yield db.b.save(b)
-        yield db.c.save(c)
-        a["hello"] = "mike"
-        yield db.a.save(a)
-        result_a = yield db.a.find_one()
-        result_b = yield db.b.find_one()
-        result_c = yield db.c.find_one()
-
-        self.assertEqual(a, result_a)
-        self.assertEqual(a, result_b["test"])
-        self.assertEqual(a, result_c["another test"]["test"])
-        self.assertEqual(b, result_b)
-        self.assertEqual(b, result_c["another test"])
-        self.assertEqual(c, result_c)
-
-    # SCRAM-SHA-1 is slow, install backports.pbkdf2 for speed.
-    @gen_test(timeout=30)
-    def test_authenticate(self):
-        # self.db is logged in as root.
-        with ignore_deprecations():
-            yield self.db.add_user("mike", "password")
-
-        client = motor.MotorClient(env.host, env.port,
-                                   **self.get_client_kwargs())
-        db = client.motor_test
-        try:
-            # Authenticate many times at once to test concurrency.
-            yield [db.authenticate("mike", "password") for _ in range(10)]
-
-            # Just make sure there are no exceptions here.
-            yield db.remove_user("mike")
-            yield db.logout()
-            info = yield self.db.command("usersInfo", "mike")
-            users = info.get('users', [])
-            self.assertFalse("mike" in [u['user'] for u in users])
-
-        finally:
-            yield remove_all_users(self.db)
-            test.env.sync_cx.close()
-
     @gen_test
     def test_validate_collection(self):
         db = self.db
