@@ -33,7 +33,7 @@ from test.test_environment import db_user, db_password, env
 from test.tornado_tests import MotorReplicaSetTestBase, MotorTest
 from test.utils import one, get_primary_pool
 
-from motor.motor_py3_compat import text_type
+from motor.motor_py2_compat import text_type
 
 
 class MotorReplicaSetTest(MotorReplicaSetTestBase):
@@ -52,23 +52,6 @@ class MotorReplicaSetTest(MotorReplicaSetTestBase):
         with self.assertRaises(pymongo.errors.ConnectionFailure):
             yield client.admin.command('ismaster')
 
-        # Test with a callback.
-        (result, error), _ = yield gen.Task(client.admin.command, 'ismaster')
-        self.assertEqual(None, result)
-        self.assertTrue(isinstance(error, pymongo.errors.ConnectionFailure))
-
-    @gen_test
-    def test_socketKeepAlive(self):
-        # Connect.
-        yield self.rsc.server_info()
-        ka = get_primary_pool(self.rsc).opts.socket_keepalive
-        self.assertTrue(ka)
-
-        client = self.motor_rsc(socketKeepAlive=False)
-        yield client.server_info()
-        ka = get_primary_pool(client).opts.socket_keepalive
-        self.assertFalse(ka)
-
     @gen_test
     def test_auth_network_error(self):
         if not test.env.auth:
@@ -82,9 +65,12 @@ class MotorReplicaSetTest(MotorReplicaSetTestBase):
 
         # Simulate an authenticate() call on a different socket.
         credentials = pymongo.auth._build_credentials_tuple(
-            'DEFAULT', 'admin',
-            text_type(db_user), text_type(db_password),
-            {})
+            'DEFAULT',
+            'admin',
+            text_type(db_user),
+            text_type(db_password),
+            {},
+            'admin')
 
         c.delegate._cache_credentials('test', credentials, connect=False)
 
