@@ -16,6 +16,7 @@ from __future__ import unicode_literals
 
 from collections import defaultdict
 
+from bson import SON
 from pymongo import monitoring
 
 """Utilities for testing Motor with any framework."""
@@ -68,7 +69,7 @@ def get_primary_pool(client):
 
 
 # Ignore auth commands like saslStart, so we can assert lsid is in all commands.
-class SessionTestListener(monitoring.CommandListener):
+class TestListener(monitoring.CommandListener):
     def __init__(self):
         self.results = defaultdict(list)
 
@@ -84,12 +85,27 @@ class SessionTestListener(monitoring.CommandListener):
         if not event.command_name.startswith('sasl'):
             self.results['failed'].append(event)
 
-    def first_command_started(self):
+    def first_command_started(self, name=None):
         assert len(self.results['started']) >= 1, (
             "No command-started events")
 
-        return self.results['started'][0]
+        if name:
+            for result in self.results['started']:
+                if result.command_name == name:
+                    return result
+        else:
+            return self.results['started'][0]
 
 
 def session_ids(client):
     return [s.session_id for s in client.delegate._topology._session_pool]
+
+
+def create_user(authdb, user, pwd=None, roles=None, **kwargs):
+    cmd = SON([('createUser', user)])
+    # X509 doesn't use a password
+    if pwd:
+        cmd['pwd'] = pwd
+    cmd['roles'] = roles or ['root']
+    cmd.update(**kwargs)
+    return authdb.command(cmd)
